@@ -4,7 +4,7 @@ use warnings;
 
 package Art::World {
 
-  our $VERSION = '0.18_04';
+  our $VERSION = '0.18_06';
 
   use Zydeco
     authority => 'cpan:SMONFF',
@@ -22,16 +22,18 @@ package Art::World {
 
   use feature qw( postderef );
   no warnings qw( experimental::postderef );
-  use Carp;
+  use Carp qw( carp cluck );
   use utf8;
   use Config::Tiny;
   use DDP;
   use Faker;
   use List::Util qw( max any );
   use Math::Round;
+  use Types::XSD qw( DateTime );
 
   role Abstraction {
     has discourse ( type => Str );
+    # TODO should only be a table in the database
     has file! ( type => ArrayRef[ Idea ], );
     has idea! ( type => Str, is => rw,  trigger => true );
     has process ( type => ArrayRef );
@@ -85,11 +87,52 @@ package Art::World {
     }
   }
 
-  role Event with Fame {
-    has place;
-    has datetime;
+  role Crud {
+    has dbh! ( type => InstanceOf[ 'Art::World::Model' ], builder => true, lazy => true );
+    has db! ( type => HashRef[ Str ], builder => true, lazy => true );
+
+    method insert ( Str $table, HashRef $attributes ) {
+
+      # TODO IT MUST BE TESTED
+      unless ( $self->does( 'Art::World::Unserializable' )) {
+        try {
+          my $row = $self->dbh->insert( ucfirst lc $table, $attributes);
+        } catch {
+          cluck 'You tried to insert to ' . $table . ' but this table doesn\'t exist';
+        }
+      }
+    }
+
+    method _build_dbh {
+      use DBI;
+      use Teng;
+      use Teng::Schema::Loader;
+      my $dbh = Teng::Schema::Loader->load(
+        dbh       => DBI->connect(
+          'dbi:' . $self->db->{ kind } .
+          ':dbname=' . $self->db->{ name },
+          '', '' ),
+        namespace => __PACKAGE__ . '::Model'
+       );
+      say ref $dbh;
+      p $dbh;
+      return $dbh;
+    }
+
+    method _build_db {
+      return {
+        name => $self->config->{ DB }->{ NAME },
+        kind => $self->config->{ DB }->{ KIND },
+       };
+    }
+  }
+
+  role Event {
+    has place ( type => Place );
+    has datetime ( type => DateTime );
     # "guests"
-    has participants;
+    has participant ( type => ArrayRef[ Agent ] );
+    has title ( type => Str, is => ro );
   }
 
   role Exhibit {
@@ -125,26 +168,28 @@ package Art::World {
     }
 
     multi method bump_fame( PositiveNum $gain ) {
-      say "PositiveNum";
       $self->$update_fame( $gain );
     }
 
     multi method bump_fame( NegativeNum $loss ) {
-      say "NegativeNum";
       $self->$update_fame( $loss )
     }
 
     multi method bump_fame {
-      say "No num";
       $self->$update_fame( $self->config->{ FAME }->{ DEFAULT_BUMP });
     }
   }
 
+  # From the documentation
+  #
+  # A `Place` must `invite()` all kind of `Agents` that produce `Work` or other kind
+  # of valuable social interactions. Depending of the total `reputation` of the
+  # `Place` it will determine it's `underground` status: when going out of the
+  # `underground`, it will become an institution.
+  #
   role Invitation {
-    multi method invite ( ArrayRef[ Agent ] *people  ) {
-
-    }
 
+    # In case a group of Agents are invited, for an Event like a performance, a concert
     multi method invite ( ArrayRef[ Agent ] *people,  Event *event  ) {
 
     }
@@ -178,6 +223,8 @@ package Art::World {
   }
 
   role Space {
+    # Like a small space or a large space
+    # Could limit the number of person coming
     has space;
   }
 
@@ -207,20 +254,33 @@ package Art::World {
     }
 
     class Opening with Event, Fame {
-      has treat;
+      has treat ( type => ArrayRef[ Str ]);
       has smalltalk;
+
+      # TODO must take as parameter what is served
+      method serve {
+        return $self->treat->@*;
+      }
     }
 
     class Sex with Event;
 
-    class Playground {
+    class Playground with Crud {
 
       class Collective;
+
       class Magazine {
+
         has reader;
+        has writer ( type => ArrayRef[ Agent ] );
+
+        method publish {};
       }
 
-      class Place with Space, Fame, Invitation {
+      class Place with Fame, Invitation, Space {
+
+        # Like, dunno, a city ? Could be in a Geo role I guess
+        has location ( type => Str );
 
         class Institution {
 
@@ -232,10 +292,6 @@ package Art::World {
             has owner;
             has money;
 
-            # Should be moved to an Opening role
-            method serve {
-              say "What would you drink?";
-            }
           }
 
           class Museum with Exhibit;
@@ -256,7 +312,7 @@ package Art::World {
 
     }
 
-    class Agent with Active, Fame {
+    class Agent with Active, Crud, Fame {
       # Should be required but will be moved to the Crudable area
       has id         ( type => Int );
       has name!      ( type => Str );
@@ -266,6 +322,8 @@ package Art::World {
       # TODO In case the networker is a Manager, the reputation bump
       # should be higher
       method networking( ArrayRef $people ) {
+
+        $self->dbh;
         my $highest_reputation = max map { $_-> reputation } $people->@*;
         my $bump = $highest_reputation > 1 ?
           round( $highest_reputation * $self->config->{ FAME }->{ BUMP_COEFFICIENT } / 100) :
@@ -341,9 +399,9 @@ package Art::World {
         method write( Theory $catalog ) { }
       }
 
-      class Director with Manager { }
+      class Director with Manager;
 
-      class Public {
+      class Public with Unserializable? {
         method visit {
           say "I visited";
         }
@@ -388,6 +446,8 @@ package Art::World {
     }
 
     # Looks like the Art::World::Meta toolkit
+    # See also https://metacpan.org/pod/Class::MOP::Class for extreme cases
+    # my $meta = $self->meta;
     class Meta {
       # TODO See also Zydeco's $class object
       method get_class( Object $klass ) {
@@ -403,7 +463,7 @@ package Art::World {
           'Moo'->_constructor_maker_for(
             $self->get_class( $claxx )
               )->all_attribute_specs
-                     });
+            });
       }
 
       method generate_discourse( ArrayRef $buzz = [] ) {
@@ -470,6 +530,27 @@ as somebody in the art world that buy but doesn't sale.
 
 If it's collectionable, it can go to a C<Collector> collection or in a C<Museum>.
 
+=head2 Crud
+
+The C<Crud> role makes possible to serialize most of the entities. For this to be
+possible, they need to have a corresponding table in the database, plus they
+need to not have a C<unserializable?> tag attribute. A Zydeco tag role is a role
+without behavior that can only be checked using C<does>.
+
+The Crud role is made possible thanks to L<Teng>, a simple DBI wrapper and O/R
+Mapper. C<Teng> is used through it's L<schema loader|Teng::Schema::Loader> that
+directly instanciate an object schema from the database.
+
+When trying to update a value, it is necessary to pass a table name and a
+hashref of attributes. The table name is not case sensitive.
+
+Note that it is extremely important that the testing of the database should be
+done in the C<t/crud.t> file. If you want to test database or model
+functionnalities in other tests, remind to create objects by specifiying the
+config parameter that use the C<test.conf> file: only the database referenced in
+C<test.conf> will be available on cpantesters and C<art.db> wont be available there,
+only C<t/test.db> is available.
+
 =head2 Event
 
 All the necessary attributes and methodes for having fun between Art::world's Agents.
@@ -670,10 +751,10 @@ Yet another kind of C<Place>, an institution with a lot of L<C<Artworks>|Art::Wo
 
 =head2 Place
 
-A `Place` must `invite()` all kind of `Agents` that produce `Work` or other kind
-of valuable social interactions. Depending of the total `reputation` of the
-`Place` it will determine it's `underground` status: when going out of the
-`underground`, it will become an institution.
+A C<Place> must C<invite()> all kind of C<Agents> that produce C<Work> or other kind
+of valuable social interactions. Depending of the total C<reputation> of the
+C<Place> it will determine it's C<underground> status: when going out of the
+C<underground>, it will become an institution.
 
 =head2 Playground
 
@@ -726,36 +807,32 @@ Returns only attributes that are set for a particular object.
 
 Returns even non-set attributes for a particular object.
 
-=head1 AUTHORS
+=head1 AUTHOR
 
 Sébastien Feugère <sebastien@feugere.net>
 
-=head2 Contributors
+=head2 ACKNOWLEDGEMENTS
 
-=over 2
+Thanks to everyone who has contributed to ack in any way, including Adrien
+Lucca, Toby Inkster, Ezgi Göç, Pierre Aubert, Seb. Hu-Rillettes, Joseph Balicki,
+Nicolas Herubel and Nadia Boursin-Piraud.
 
-Ezgi Göç
+This project was made possible by the greatness of the L<Zydeco|https://zydeco.toby.ink/> .
 
-Joseph Balicki
+=head2 MOTIVATIONS
 
-Nadia Boursin-Piraud
-
-Nicolas Herubel
-
-Pierre Aubert
-
-Seb. Hu-Rillettes
-
-Toby Inkster
-
-=back
-
-=head1 ACKNOWLEDGMENT
-
-This project was made possible by the greatness of L<Zydeco|https://zydeco.toby.ink/>.
+This project is a self-teaching experiment around the modern Perl
+object-oriented programming toolkit. In late 2020, I read a bit about
+L<Corrina|https://github.com/Ovid/Cor/wiki> and immediatly wanted to restart and
+experiment my old C<Art::World> project. While searching for something somewhat
+close to Corrina, since Corrina was only a draft and RFC, I discovered the
+incredible Zydeco project by Toby Inkster and immediatly felt very enthusiastic
+to use it to implement my idea. I hope it is a decent example of the
+possibilities this wonderful framework make possible.
 
 =head1 COPYRIGHT AND LICENSE
 
 Copyright 2006-2021 Sebastien Feugère
 
-This library is free software; you can redistribute it and/or modify it under the Artistic License 2.0.
+This library is free software; you can redistribute it and/or modify it under
+the Artistic License 2.0.
